@@ -1,7 +1,7 @@
 import time
 from unittest import TestCase
 from sqlalchemy.ext.serializer import loads, dumps
-from copy import deepcopy
+
 from freezegun import freeze_time
 
 from recipe import Recipe
@@ -196,6 +196,54 @@ class TestSQLAlchemyBuilder(TestBase):
             self.assertEqual(to_sql(expr), expected_sql)
 
 
+class TestSQLAlchemyBuilderConvertDates(TestBase):
+    def test_enforce_convert_dates(self):
+        """Enforce aggregation will wrap the function in a sum if no aggregation was seen"""
+
+        good_examples = """
+        [test_date]                       -> date_trunc('year', datatypes.test_date)
+        test_date                         -> date_trunc('year', datatypes.test_date)
+        coalesce([test_date], date("2020-01-01"))   -> coalesce(date_trunc('year', datatypes.test_date), '2020-01-01')
+        """
+        for field, expected_sql in self.examples(good_examples):
+            expr, _ = self.builder.parse(
+                field,
+                enforce_aggregation=True,
+                debug=True,
+                convert_dates_with="year_conv",
+            )
+            self.assertEqual(to_sql(expr), expected_sql)
+
+        good_examples = """
+        [test_date]                       -> date_trunc('month', datatypes.test_date)
+        test_date                         -> date_trunc('month', datatypes.test_date)
+        coalesce([test_date], date("2020-01-01"))   -> coalesce(date_trunc('month', datatypes.test_date), '2020-01-01')
+        """
+        for field, expected_sql in self.examples(good_examples):
+            expr, _ = self.builder.parse(
+                field,
+                enforce_aggregation=True,
+                debug=True,
+                convert_dates_with="month_conv",
+            )
+            self.assertEqual(to_sql(expr), expected_sql)
+
+        # If the date conversion doesn't exist, don't convert
+        good_examples = """
+        [test_date]                       -> datatypes.test_date
+        test_date                         -> datatypes.test_date
+        coalesce([test_date], date("2020-01-01"))   -> coalesce(datatypes.test_date, '2020-01-01')
+        """
+        for field, expected_sql in self.examples(good_examples):
+            expr, _ = self.builder.parse(
+                field,
+                enforce_aggregation=True,
+                debug=True,
+                convert_dates_with="a_potato",
+            )
+            self.assertEqual(to_sql(expr), expected_sql)
+
+
 class TestDataTypesTable(TestBase):
     def test_fields_and_addition(self):
         """These examples should all succeed"""
@@ -614,6 +662,7 @@ class TestAggregations(TestBase):
         count_distinct(department)   -> count(DISTINCT datatypes.department)
         count_distinct(department = "MO" AND score > 20) -> count(DISTINCT (datatypes.department = 'MO' AND datatypes.score > 20))
         count_distinct(if(department = "MO" AND score > 20, department)) -> count(DISTINCT CASE WHEN (datatypes.department = 'MO' AND datatypes.score > 20) THEN datatypes.department END)
+        count(IF(department = "MO" AND score > 20, department)) -> count(CASE WHEN (datatypes.department = 'MO' AND datatypes.score > 20) THEN datatypes.department END)
         count(*)                     -> count(*)
         """
 
